<?php

namespace App\Http\Controllers\V1;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Post;
use Illuminate\Http\Request;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

class HomeController extends Controller
{

    
    public function categories(Request $request)
    {

        $lang = $request->lang ?? 'es';
        $sort_by = 'asc';
        $limit = 10;

        $data = Category::query();

        if($request->has('search')){
            $search = $request->search;
            $data->where('title', 'like', '%'.$search.'%');
        }

        if($request->has('id') && $request->id){
            $data->where('id',$request->id);
        }

        if($request->has('sort_by') && $request->sort_by != ''){
            $sort_by = $request->sort_by;
        }

        if($request->has('order_by') && $request->order_by != null){
            $data->orderBy($request->order_by,$sort_by);
        }


        $total = $data->count();
       
        $currentPage = $request->input('page', 1);
        $offset = ($currentPage - 1) * $limit;
        $data = $data->limit($limit)->offset($offset)
        ->select([
            'id',
            'title_'.$lang.' as title',
            'is_featured',
            'status',
            'created_at',
            'updated_at'
        ])->get();

        // $data = $data->map(function ($item){
        //     $item['thumbnail_prev'] = asset($item['thumbnail']);
        //     return $item;
        // });

        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'page' => $currentPage,
                'total' => $total,
                'data' => $data,
            ],
        ],200);

    }


    public function posts(Request $request)
    {

        $lang = $request->lang ?? 'es';
        $sort_by = 'asc';
        $limit = $request->limit ?? 90;

        $data = Post::Leftjoin('categories','categories.id','=','posts.category_id');

        if($request->has('search') && $request->search != ''){
            $search = $request->search;
            $data->where('posts.title_'.$lang, 'like', '%'.$search.'%');
        }

        if($request->has('id') && $request->id){
            $data->where('posts.id',$request->id);
        }

        if($request->has('slug') && $request->slug){
            $data->where('posts.slug',$request->slug);
        }

        if($request->has('category') && $request->category){
            $data->where('posts.category_id',$request->category);
        }
        
         if($request->has('status') && $request->status){
            $data->where('posts.status',$request->status);
        }
        
        if($request->has('is_featured') && $request->is_featured){
            $data->where('posts.is_featured',$request->is_featured);
        }

        if($request->has('type') && $request->type){
            
            $data->where('posts.type',$request->type);
        }
        
         if($request->has('type_in') && $request->type_in){
            $data->whereIn('posts.type',explode(',',$request->type_in));
        }
        
        if($request->has('dctype') && $request->dctype){
            $data->where('posts.dctype',$request->dctype);
        }

        if($request->has('sort_by') && $request->sort_by != ''){
            $sort_by = $request->sort_by;
        }

        if($request->has('order_by') && $request->order_by != null){
            $data->orderBy($request->order_by,$sort_by);
        }

        $total = $data->count();
       
        $currentPage = $request->input('page', 1);
        $offset = ($currentPage - 1) * $limit;
        $data = $data->limit($limit)->offset($offset)
        ->select([
            'posts.id',
            'posts.slug',
            'categories.title_'.$lang.' as category',
            'posts.title_'.$lang.' as title',
            'posts.subtitle_'.$lang.' as subtitle',
            'posts.short_description_'.$lang.' as short_description',
            'posts.long_description_'.$lang.' as long_description',
            'posts.requirements_'.$lang.' as requirements',
            
            'posts.thumbnail_'.$lang.' as thumbnail',
            'posts.pdf_'.$lang.' as pdf',
            'posts.author_'.$lang.' as author',

            'posts.views_'.$lang.' as views',
            'posts.like_'.$lang.' as like',

            'posts.banner_'.$lang.' as banner',

            'posts.creater_'.$lang.' as creater',
            'posts.topic_'.$lang.' as topic',
            'posts.country_'.$lang.' as country',
            
            'posts.button_'.$lang.' as button',
            'posts.link_'.$lang.' as link',
            

            'posts.is_featured',
            'posts.status',
            'posts.type',
            'posts.dctype',
            'posts.category_id',
            'posts.created_at',
            'posts.updated_at'
        ])->get();

        $data = $data->map(function ($item) use($lang) {
           
            $item['thumbnail_prev'] = asset($item['thumbnail']);
            $item['banner_prev'] = asset($item['banner']);
            $item['pdf_prev'] = asset($item['pdf']);
            

            $existingDate = $item->created_at;
            $date = Carbon::parse($existingDate);
            
            $formattedDate = $date->copy()->locale($lang)->translatedFormat('d M Y');
            
            $relativeTime = $date->diffForHumans([
                'parts' => 1,
                'short' => true,
            ]);
            
            $result = "{$formattedDate} - {$relativeTime}";
            
            $item['date_formated'] = $result;

               
            $item['date'] = $formattedDate;



            return $item;
        });

        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'page' => $currentPage,
                'total' => $total,
                'data' => $data,
            ],
        ],200);
        
    }

    public function posts_by_year(Request $request)
    {

        $lang = $request->lang ?? 'es';
        $sort_by = 'asc';

        DB::statement("SET SESSION sql_mode=(SELECT REPLACE(@@sql_mode, 'ONLY_FULL_GROUP_BY', ''));");



        $data = Post::Leftjoin('categories','categories.id','=','posts.category_id');

        if($request->has('id') && $request->id){
            $data->where('posts.id',$request->id);
        }

        if($request->has('type') && $request->type){
            $data->where('posts.type',$request->type);
        }

        if($request->has('sort_by') && $request->sort_by != ''){
            $sort_by = $request->sort_by;
        }

        if($request->has('order_by') && $request->order_by != null){
            $data->orderBy($request->order_by,$sort_by);
        }

        $data = $data->select([
            'posts.id',
            'posts.slug',
            'categories.title_'.$lang.' as category',
            'posts.title_'.$lang.' as title',
            'posts.short_description_'.$lang.' as short_description',
            'posts.long_description_'.$lang.' as long_description',
            'posts.thumbnail_'.$lang.' as thumbnail',
            'posts.pdf_'.$lang.' as pdf',
            'posts.author_'.$lang.' as author',
            'posts.is_featured',
            'posts.status',
            'posts.type',
            'posts.category_id',
            'posts.created_at',
            'posts.updated_at',
            DB::raw('YEAR(posts.created_at) as year')
        ])
        ->groupBy(DB::raw('YEAR(posts.created_at)'), 'posts.id')
        ->orderBy('year')
        ->get();

        
        $data = $data->map(function ($item){
            $item['thumbnail_prev'] = asset($item['thumbnail']);
            return $item;
        });


        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'data' => $data->groupBy('year'),
            ],
        ],200);
        
    }
    
    
    
    public function events(Request $request)
    {

        $lang = $request->lang ?? 'es';
        $sort_by = 'asc';
        $limit = $request->limit ?? 90;

        $data = Post::query();
        
        $data->where('posts.type','event');

        if($request->has('search') && $request->search != ''){
            $search = $request->search;
            $data->where('posts.title_'.$lang, 'like', '%'.$search.'%');
        }

        if($request->has('id') && $request->id){
            $data->where('posts.id',$request->id);
        }

        if($request->has('slug') && $request->slug){
            $data->where('posts.slug',$request->slug);
        }
        
        if($request->has('status') && $request->status){
            $data->where('posts.status',$request->status);
        }
        
        if($request->has('is_featured') && $request->is_featured){
            $data->where('posts.is_featured',$request->is_featured);
        }
        
        if($request->has('sort_by') && $request->sort_by != ''){
            $sort_by = $request->sort_by;
        }

        if($request->has('order_by') && $request->order_by != null){
            $data->orderBy($request->order_by,$sort_by);
        }

        $total = $data->count();
       
        $currentPage = $request->input('page', 1);
        $offset = ($currentPage - 1) * $limit;
        $data = $data->limit($limit)->offset($offset)
        ->select([
            'posts.id',
            'posts.slug',
            
            'posts.title_'.$lang.' as title',
            'posts.subtitle_'.$lang.' as subtitle',
            'posts.short_description_'.$lang.' as short_description',
            'posts.long_description_'.$lang.' as long_description',
            'posts.thumbnail_'.$lang.' as thumbnail',
            'posts.banner_'.$lang.' as banner',
            'posts.button_'.$lang.' as button',
            'posts.is_featured',
            'posts.status',
            'posts.type',
            'posts.start_date',
            'posts.end_date',
            'posts.created_at',
            'posts.updated_at'
        ])->get();

        $data = $data->map(function ($item) use($lang) {
           
            $item['thumbnail_prev'] = asset($item['thumbnail']);
            $item['banner_prev'] = asset($item['banner']);
       
            

            $existingDate = $item->created_at;
            $date = Carbon::parse($existingDate);
            $formattedDate = $date->copy()->locale($lang)->translatedFormat('d M Y');
            $relativeTime = $date->diffForHumans([
                'parts' => 1,
                'short' => true,
            ]);
            $result = "{$formattedDate} - {$relativeTime}";
            $item['date_formated'] = $result;
            $item['date'] = $formattedDate;
            
            
             $month = Carbon::parse($item->start_date);
             $item['month'] = $month->copy()->locale($lang)->translatedFormat('M');
             
             
             $sdate = Carbon::parse($item->start_date);
             $item['sdate'] = $sdate->copy()->locale($lang)->translatedFormat('d');
             
             $edate = Carbon::parse($item->end_date);
             $item['edate'] = $edate->copy()->locale($lang)->translatedFormat('d');
            
            
            
            return $item;
            
        });

        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'page' => $currentPage,
                'total' => $total,
                'data' => $data,
            ],
        ],200);
        
    }
    
    
    
    
     public function document(Request $request)
    {

        $lang = $request->lang ?? 'es';
        $sort_by = 'asc';
        $limit = $request->limit ?? 90;
        
        
        $data = Post::where('posts.type','document');
        
            if($request->has('search') && $request->search != ''){
             $search = $request->search;
             $data->where(function ($q) use ($search, $lang) {
                $q->where("title_$lang", 'like', "%$search%")
                    ->orWhere("short_description_$lang", 'like', "%$search%");
            });
        }
        
        if($request->has('year') && $request->year){
            $data->whereYear('posts.created_at', $request->year);
        }
        
        if($request->has('topic') && $request->topic){
            $data->where('posts.topic_'.$lang,$request->topic);
        }
        
        if($request->has('country') && $request->country){
            $data->where('posts.country_'.$lang,$request->country);
        }
        
        if($request->has('id') && $request->id){
            $data->where('posts.id',$request->id);
        }

        if($request->has('slug') && $request->slug){
            $data->where('posts.slug',$request->slug);
        }
        
        if($request->has('sort_by') && $request->sort_by != ''){
            $sort_by = $request->sort_by;
        }

        if($request->has('order_by') && $request->order_by != null){
            $data->orderBy($request->order_by,$sort_by);
        }
        
        $total = (clone $data)->count();
        $currentPage = $request->input('page', 1);
        $offset = ($currentPage - 1) * $limit;
        
        $data = $data
        ->limit($limit)
        ->offset($offset)
        ->select([
            'posts.id',
            'posts.slug',
        
            'posts.title_'.$lang.' as title',
            'posts.topic_'.$lang.' as topic',
            'posts.country_'.$lang.' as country',
            'posts.short_description_'.$lang.' as short_description',
            'posts.thumbnail_'.$lang.' as thumbnail',
            'posts.pdf_'.$lang.' as pdf',
            
            'posts.views_'.$lang.' as views',
            'posts.like_'.$lang.' as like',
            
            'posts.created_at',
            'posts.updated_at'
        ])->get();


        $data = $data->map(function ($item) use($lang) {
            
            $date = Carbon::parse($item->created_at);
            $item['date'] = $date->copy()->locale($lang)->translatedFormat('d M Y');
            return $item;
            
        });

        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'page' => $currentPage,
                'total' => $total,
                'data' => $data,
            ],
        ],200);
        
        
    }
    
    
    
    public function document_filters(Request $request){
        
        $lang = $request->lang ?? 'es';
        $couuntry = Post::where('posts.type','document')->pluck('country_'.$lang)->toArray();
        $topic = Post::where('posts.type','document')->pluck('topic_'.$lang)->toArray();
        $years = Post::where('posts.type', 'document')
        ->select(DB::raw('YEAR(created_at) as year'))
        ->distinct()
        ->orderBy('year', 'desc') 
        ->pluck('year')
        ->toArray();
        
    
        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'years' => $years,
                'topics' => $topic,
                'country' => $couuntry,
            ],
        ],200);
    
        
         
    }
    
    
    
}