<?php

namespace App\Http\Controllers\V1;

use App\Http\Controllers\Controller;
use App\Models\Post;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use Carbon\Carbon;

class PostController extends Controller
{
    
    
    public function document(Request $request)
    { 
        
        $lang = $request->lang ?? 'en';
        $sort_by = $request->sort_by ?? 'desc';
        $order_by = $request->order_by ?? 'id';
        $limit = $request->limit ?? 10;
        $page = $request->page ?? 1;

        //Query
        $data = Post::query();


        if($request->has('search') && $request->search != ''){
             $search = $request->search;
             $data->where(function ($q) use ($search, $lang) {
                $q->where("title_$lang", 'like', "%$search%")
                    ->orWhere("country_$lang", 'like', "%$search%")
                    ->orWhere("topic_$lang", 'like', "%$search%")
                    ->orWhere('created_at', 'like', "%$search%")
                    ->orWhere('id', 'like', "%$search%");
            });
        }
        
        $data = $data->where('posts.type','document');

        if($request->has('id') && $request->id){
           $data = $data->where('posts.id',$request->id);
        }

        if($request->has('slug') && $request->slug){
          $data = $data->where('posts.slug',$request->slug);
        }

        if($request->has('category') && $request->category){
          $data  = $data->where('posts.category_id',$request->category);
        }

        $total = (clone $data)->count();
        $data = $data->limit($limit)
        ->offset(($page - 1) * $limit)
        ->orderBy($order_by,$sort_by)
        ->select([
            'posts.id',
            'posts.slug',
            'posts.title_'.$lang.' as title',
            'posts.short_description_'.$lang.' as short_description',
            'posts.thumbnail_'.$lang.' as thumbnail',
            'posts.pdf_'.$lang.' as pdf',
            'posts.views_'.$lang.' as views',
            'posts.like_'.$lang.' as like',
            'posts.topic_'.$lang.' as topic',
            'posts.country_'.$lang.' as country',
            'posts.created_at',
            'posts.updated_at'
        ])->get();
        
        $data->map(function ($item){
            $existingDate = $item->created_at;
            $date = Carbon::parse($existingDate);
            $item['date'] = $date->translatedFormat('Y-m-d');
            return $item;
        });

        $paginations = [];
        for ($i=1; $i < ceil($total / $limit); $i++) { 
          array_push($paginations,$i);    
        }


        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'total' => $total,
                'from' => ($page - 1) * $limit + 1,
                'to' => min($page * $limit, $total),
                'page' => $page,
                'last_page' => ceil($total / $limit),
                'data' => $data,
                'links' => $paginations,
            ],
        ],200);

    }
    
    
    
    public function pdf(Request $request)
    { 
        
        $lang = $request->lang ?? 'en';
        $sort_by = $request->sort_by ?? 'desc';
        $order_by = $request->order_by ?? 'id';
        $limit = $request->limit ?? 10;
        $page = $request->page ?? 1;


        //Query
        $data = Post::query();
        
        $data->where('posts.type','pdf');
        
        if($request->has('search') && $request->search != ''){
             $search = $request->search;
             $data->where(function ($q) use ($search, $lang) {
                $q->where("title_$lang", 'like', "%$search%")
                    ->orWhere("author_$lang", 'like', "%$search%")
                    ->orWhere("creater_$lang", 'like', "%$search%")
                    ->orWhere('created_at', 'like', "%$search%")
                    ->orWhere('id', 'like', "%$search%");
            });
        }

        if($request->has('id') && $request->id){
            $data->where('posts.id',$request->id);
        }

        if($request->has('slug') && $request->slug){
            $data->where('posts.slug',$request->slug);
        }
        
        $total = $data->count(); 
        $data = $data->limit($limit)
        ->offset(($page - 1) * $limit)
        ->orderBy($order_by,$sort_by)
        ->select([
            'posts.id',
            'posts.slug',
            'posts.title_'.$lang.' as title',
            'posts.short_description_'.$lang.' as short_description',
            'posts.long_description_'.$lang.' as long_description',
            'posts.thumbnail_'.$lang.' as thumbnail',
            'posts.pdf_'.$lang.' as pdf',
            'posts.author_'.$lang.' as author',
            'posts.views_'.$lang.' as views',
            'posts.like_'.$lang.' as like',
            'posts.banner_'.$lang.' as banner',
            'posts.creater_'.$lang.' as creater',
            'posts.is_featured',
            'posts.status',
            'posts.created_at',
            'posts.updated_at'
        ])->get();

        $data->map(function ($item){
            $existingDate = $item->created_at;
            $date = Carbon::parse($existingDate);
            $item['date'] = $date->translatedFormat('Y-m-d');
            return $item;
        });

        $paginations = [];
        for ($i=1; $i < ceil($total / $limit); $i++) { 
          array_push($paginations,$i);    
        }

        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'total' => $total,
                'from' => ($page - 1) * $limit + 1,
                'to' => min($page * $limit, $total),
                'page' => $page,
                'last_page' => ceil($total / $limit),
                'data' => $data,
                'links' => $paginations,
            ],
        ],200);

    }
    
    
    public function event(Request $request)
    { 
        
        $lang = $request->lang ?? 'en';
        $sort_by = $request->sort_by ?? 'desc';
        $order_by = $request->order_by ?? 'id';
        $limit = $request->limit ?? 10;
        $page = $request->page ?? 1;


        //Query
        $data = Post::where('posts.type','event');

        if($request->has('search') && $request->search != ''){
            $search = $request->search;
            $data->where('posts.title_'.$lang, 'like', '%'.$search.'%')
            ->orWhere('posts.subtitle_'.$lang, 'like', '%'.$search.'%')
            ->orWhere('posts.created_at','like','%'.$search.'%');
        }

        if($request->has('id') && $request->id){
            $data->where('posts.id',$request->id);
        }

        if($request->has('slug') && $request->slug){
            $data->where('posts.slug',$request->slug);
        }
        
        $total = $data->count(); 
        $data = $data->limit($limit)
        ->offset(($page - 1) * $limit)
        ->orderBy($order_by,$sort_by)
        ->select([
            'posts.id',
            'posts.slug',
            'posts.title_'.$lang.' as title',
            'posts.subtitle_'.$lang.' as subtitle',
            
            
            'posts.short_description_'.$lang.' as short_description',
            'posts.long_description_'.$lang.' as long_description',
            'posts.thumbnail_'.$lang.' as thumbnail',
            'posts.button_'.$lang.' as button',
            'posts.link_'.$lang.' as link',
            'posts.is_featured',
            'posts.status',
            'posts.type',
            'posts.start_date',
            'posts.end_date',
            'posts.created_at',
            'posts.updated_at'
        ])->get();

        $data->map(function ($item){
            if(isset($item['thumbnail'])){
                $item['thumbnail_prev'] = asset($item['thumbnail']);
            }    
            
            $existingDate = $item->created_at;
            $date = Carbon::parse($existingDate);
            $item['date'] = $date->translatedFormat('Y-m-d');
            
            return $item;
        });

        $paginations = [];
        for ($i=1; $i < ceil($total / $limit); $i++) { 
          array_push($paginations,$i);    
        }

        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'total' => $total,
                'from' => ($page - 1) * $limit + 1,
                'to' => min($page * $limit, $total),
                'page' => $page,
                'last_page' => ceil($total / $limit),
                'data' => $data,
                'links' => $paginations,
            ],
        ],200);

    }
    
    
    
    public function webinar(Request $request)
    { 
        
        $lang = $request->lang ?? 'en';
        $sort_by = $request->sort_by ?? 'desc';
        $order_by = $request->order_by ?? 'id';
        $limit = $request->limit ?? 10;
        $page = $request->page ?? 1;


        //Query
        $data = Post::query();

        // if($request->has('type')){
            $data->where('posts.type','webinar');
        // }
        
        // if($request->has('dctype')){
            // $data->where('posts.dctype',$request->dctype);
        // }

        if($request->has('search') && $request->search != ''){
            $search = $request->search;
            $data->where('posts.title_'.$lang, 'like', '%'.$search.'%')
            ->orWhere('posts.subtitle_'.$lang, 'like', '%'.$search.'%')
            // ->orWhere('posts.dctype', 'like', '%'.$search.'%')
            ->orWhere('posts.created_at','like','%'.$search.'%');
        }

        if($request->has('id') && $request->id){
            $data->where('posts.id',$request->id);
        }

        if($request->has('slug') && $request->slug){
            $data->where('posts.slug',$request->slug);
        }

        // if($request->has('category') && $request->category){
            // $data->where('posts.category_id',$request->category);
        // }

        $total = $data->count(); 
        $data = $data->limit($limit)
        ->offset(($page - 1) * $limit)
        ->orderBy($order_by,$sort_by)
        ->select([
            'posts.id',
            'posts.slug',
            'posts.title_'.$lang.' as title',
            'posts.subtitle_'.$lang.' as subtitle',
            'posts.short_description_'.$lang.' as short_description',
            'posts.long_description_'.$lang.' as long_description',
            'posts.thumbnail_'.$lang.' as thumbnail',
            'posts.button_'.$lang.' as button',
            'posts.link_'.$lang.' as link',
            'posts.dctype',
            'posts.is_featured',
            'posts.status',
            'posts.type',
            'posts.created_at',
            'posts.updated_at'
        ])->get();

        $data->map(function ($item){
            if(isset($item['thumbnail'])){
                $item['thumbnail_prev'] = asset($item['thumbnail']);
            }    
            
            $existingDate = $item->created_at;
            $date = Carbon::parse($existingDate);
            $item['date'] = $date->translatedFormat('Y-m-d');
            
            return $item;
        });

        $paginations = [];
        for ($i=1; $i < ceil($total / $limit); $i++) { 
          array_push($paginations,$i);    
        }

        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'total' => $total,
                'from' => ($page - 1) * $limit + 1,
                'to' => min($page * $limit, $total),
                'page' => $page,
                'last_page' => ceil($total / $limit),
                'data' => $data,
                'links' => $paginations,
            ],
        ],200);

    }
    
    public function course(Request $request)
    { 
        
        $lang = $request->lang ?? 'en';
        $sort_by = $request->sort_by ?? 'desc';
        $order_by = $request->order_by ?? 'id';
        $limit = $request->limit ?? 10;
        $page = $request->page ?? 1;


        //Query
        $data = Post::query();

        if($request->has('type')){
            $data->where('posts.type','course');
        }
        
        if($request->has('dctype')){
            $data->where('posts.dctype',$request->dctype);
        }

        if($request->has('search') && $request->search != ''){
            $search = $request->search;
            $data->where('posts.title_'.$lang, 'like', '%'.$search.'%')
            ->orWhere('posts.subtitle_'.$lang, 'like', '%'.$search.'%')
            ->orWhere('posts.dctype', 'like', '%'.$search.'%')
            ->orWhere('posts.created_at','like','%'.$search.'%');
        }

        if($request->has('id') && $request->id){
            $data->where('posts.id',$request->id);
        }

        if($request->has('slug') && $request->slug){
            $data->where('posts.slug',$request->slug);
        }

        if($request->has('category') && $request->category){
            $data->where('posts.category_id',$request->category);
        }

        $total = $data->count(); 
        $data = $data->limit($limit)
        ->offset(($page - 1) * $limit)
        ->orderBy($order_by,$sort_by)
        ->select([
            'posts.id',
            'posts.slug',
            'posts.title_'.$lang.' as title',
            'posts.subtitle_'.$lang.' as subtitle',
            'posts.short_description_'.$lang.' as short_description',
            'posts.requirements_'.$lang.' as requirements',
            'posts.long_description_'.$lang.' as long_description',
            'posts.thumbnail_'.$lang.' as thumbnail',
            'posts.author_'.$lang.' as author',
            'posts.creater_'.$lang.' as creater',
            'posts.button_'.$lang.' as button',
            'posts.link_'.$lang.' as link',
            'posts.dctype',
            'posts.is_featured',
            'posts.status',
            'posts.type',
            'posts.created_at',
            'posts.updated_at'
        ])->get();

        $data->map(function ($item){
            if(isset($item['thumbnail'])){
                $item['thumbnail_prev'] = asset($item['thumbnail']);
            }    
            
            $existingDate = $item->created_at;
            $date = Carbon::parse($existingDate);
            $item['date'] = $date->translatedFormat('Y-m-d');
            
            return $item;
        });

        $paginations = [];
        for ($i=1; $i < ceil($total / $limit); $i++) { 
          array_push($paginations,$i);    
        }

        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'total' => $total,
                'from' => ($page - 1) * $limit + 1,
                'to' => min($page * $limit, $total),
                'page' => $page,
                'last_page' => ceil($total / $limit),
                'data' => $data,
                'links' => $paginations,
            ],
        ],200);

    }



    
    /**
     * Show the profile for a given user.
     */
    public function index(Request $request)
    { 

        $lang = $request->lang ?? 'en';
        $sort_by = $request->sort_by ?? 'desc';
        $order_by = $request->order_by ?? 'id';
        $limit = $request->limit ?? 10;
        $page = $request->page ?? 1;

        //Query
        $data = Post::Leftjoin('categories','categories.id','=','posts.category_id');

        if($request->has('type')){
            $data->where('posts.type','post');
        }

        if($request->has('search') && $request->search != ''){
            $search = $request->search;
            $data->where('posts.title_'.$lang, 'like', '%'.$search.'%')
            ->orWhere('categories.title_'.$lang, 'like', '%'.$search.'%')
            ->orWhere('posts.created_at','like','%'.$search.'%');
        }

        if($request->has('id') && $request->id){
            $data->where('posts.id',$request->id);
        }

        if($request->has('slug') && $request->slug){
            $data->where('posts.slug',$request->slug);
        }

        if($request->has('category') && $request->category){
            $data->where('posts.category_id',$request->category);
        }
        
        

        $total = $data->count(); 
        $data = $data->limit($limit)
        ->offset(($page - 1) * $limit)
        ->orderBy($order_by,$sort_by)
        ->select([
            'posts.id',
            'posts.slug',
            'categories.title_'.$lang.' as category',
            'posts.title_'.$lang.' as title',
            'posts.short_description_'.$lang.' as short_description',
            'posts.long_description_'.$lang.' as long_description',
            'posts.thumbnail_'.$lang.' as thumbnail',        
            'posts.is_featured',
            'posts.status',
            'posts.type',
            'posts.category_id',
            'posts.created_at',
            'posts.updated_at'
        ])->get();

        $data->map(function ($item){
            if(isset($item['thumbnail'])){
                $item['thumbnail_prev'] = asset($item['thumbnail']);
            }
            
            $existingDate = $item->created_at;
            $date = Carbon::parse($existingDate);
            $item['date'] = $date->translatedFormat('Y-m-d');
            
            return $item;
        });

        $paginations = [];
        for ($i=1; $i < ceil($total / $limit); $i++) { 
          array_push($paginations,$i);    
        }

        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'total' => $total,
                'from' => ($page - 1) * $limit + 1,
                'to' => min($page * $limit, $total),
                'page' => $page,
                'last_page' => ceil($total / $limit),
                'data' => $data,
                'links' => $paginations,
            ],
        ],200);

    }





    /*
     * Show the profile for a given user.
     */
    public function update(Request $request,$id)
    {

        $lang = $request->lang ?? 'en';
        $module = Post::where('id',$id)->first();
        if($module == false){
             $module = new Post();
        }

        if($request->has('title')){
            $module->{"title_" . $lang} = $request->title;
            $slug = strtolower($request->title);
            $slug = preg_replace('/[^a-z0-9-]/', ' ', $slug);
            $slug = preg_replace('/\s+/', '-', $slug);
            $slug = trim($slug, '-');
            $module->slug = $slug;
        }
        
        if($request->has('subtitle')){
            $module->{"subtitle_" . $lang} = $request->subtitle;
        }

        if($request->has('short_description')){
            $module->{"short_description_" . $lang} = $request->short_description;
        }
        
        

        if($request->has('long_description')){
            $module->{"long_description_" . $lang} = $request->long_description;
        }
        
         if($request->has('requirements')){
            $module->{"requirements_" . $lang} = $request->requirements;
        }
        
   

        if($request->has('thumbnail')){
            $module->{"thumbnail_" . $lang} = $request->thumbnail;
        }

        if($request->has('banner')){
            $module->{"banner_" . $lang} = $request->banner;
        }

        if($request->has('creater')){
            $module->{"creater_" . $lang} = $request->creater;
        }

        if($request->has('author')){
            $module->{"author_" . $lang} = $request->author;
        }

        if($request->has('pdf')){
            $module->{"pdf_" . $lang} = $request->pdf;
        }
        
        if($request->has('country')){
            $module->{"country_" . $lang} = $request->country;
        }
        
        if($request->has('button')){
            $module->{"button_" . $lang} = $request->button;
        }
        
         if($request->has('link')){
            $module->{"link_" . $lang} = $request->link;
        }
        
        if($request->has('topic')){
            $module->{"topic_" . $lang} = $request->topic;
        }
        
        if($request->has('category_id')){
            $module->category_id = $request->category_id;
        }

        if($request->has('status')){
            $module->status = $request->status;
        }
        
        if($request->has('dctype')){
            $module->dctype = $request->dctype;
        }
        
        
         if($request->has('start_date')){
            $module->start_date = $request->start_date;
        }
        
         if($request->has('end_date')){
            $module->end_date = $request->end_date;
        }
        
        
        if($request->has('featured')){
            $module->is_featured = $request->featured;
        }
        
        $module->type = $request->type;
        $module->save();

        return response()->json([
            "message" => "Record Updated Successfully",
            "data" => ['id' => $module->id]
        ],200);

    }



    /*
     * Show the profile for a given user.
     */
    public function destroy($id)
    {
        $module = Post::find($id);
        if($module == null){
             return response()->json(["message" => 'Record Not Found'],403);
        }
        
        $module->delete();
        return response()->json([
            "message" => 'Record Deleted Successfully',
            "data" => ['id' => $id]
        ],200);
    }


 
    
}