<?php

namespace App\Http\Controllers\V1;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use Hash;
use Carbon\Carbon;
use Illuminate\Support\Facades\Validator;

class UserController extends Controller
{
    
    
     public function users(Request $request)
    {
        

        $lang = $request->lang ?? 'en';
        $sort_by = $request->sort_by ?? 'desc';
        $order_by = $request->order_by ?? 'id';
        $limit = $request->limit ?? 10;
        $page = $request->page ?? 1;

        //Query
        $data = User::query();


        if($request->has('search') && $request->search != ''){
             $search = $request->search;
             $data->where(function ($q) use ($search, $lang) {
                $q->where("name", 'like', "%$search%")
                    ->orWhere("email", 'like', "%$search%")
                    ->orWhere('created_at', 'like', "%$search%")
                    ->orWhere('id', 'like', "%$search%");
            });
        }
        
        $data = $data->where('id','!=',1);

        if($request->has('id') && $request->id){
           $data = $data->where('id',$request->id);
        }
        
      

        $total = (clone $data)->count();
        $data = $data->limit($limit)
        ->offset(($page - 1) * $limit)
        ->orderBy($order_by,$sort_by)
        ->select([
            'id',
            'name',
            'email',
            'password',
            'permissions',
            'created_at',
        ])->get();
        
        $data->map(function ($item){
            
            $existingDate = $item->created_at;
            $date = Carbon::parse($existingDate);
            $item['date'] = $date->translatedFormat('Y-m-d');
            
            $item['permissions'] = explode(',',$item['permissions']);
            return $item;
            
        });

        $paginations = [];
        for ($i=1; $i < ceil($total / $limit); $i++) { 
          array_push($paginations,$i);    
        }


        return response()->json([
            'status' => 'success',
            "message" => "Get All Record Successfully",
            "data" =>  [
                'total' => $total,
                'from' => ($page - 1) * $limit + 1,
                'to' => min($page * $limit, $total),
                'page' => $page,
                'last_page' => ceil($total / $limit),
                'data' => $data,
                'links' => $paginations,
            ],
        ],200);

    
        
    }
    
      /*
     * Get Auth Details
     */
    public function store(Request $request)
    {
        
          $validator = Validator::make($request->all(),[
            'name' =>'required|min:6|max:255|unique:users,name', 
            'email' =>'required|email|unique:users,email', 
            'password' =>'required|min:6|max:255', 
         ]);
        
         if($validator->fails()){
             return response()->json([
                'status' => 'error',
                "message" => $validator->messages()->first(),
            ],403);
         }
         
         
        User::create([
            'name'  => $request->name,
            'email' =>  $request->email,
            'password' => Hash::make($request->password),
            'permission' => $request->permissions,
            'created_at' => '',
            'updated_at' => '',
        ]);
        
        return response()->json([
            'status' => 'success',
            "message" => "Record Created Successfull",
        ],200);
        
        
    }
    
        /*
     * Get Auth Details
     */
    public function update(Request $request,$id)
    {
          $user = User::find($id);
          
          if(!$user){
                 return response()->json([
                    'status' => 'error',
                    "message" => 'User Not Found',
                ],403);
          }
        
          $validator = Validator::make($request->all(),[
            'name' =>'required|min:6|max:255|unique:users,name,'.$user->id, 
            'email' =>'required|email|unique:users,email,'.$user->id, 
            'password' =>'nullable|min:6|max:255', 
         ]);
        
         if($validator->fails()){
             return response()->json([
                'status' => 'error',
                "message" => $validator->messages()->first(),
            ],403);
         }
         
         $user->name = $request->name;
         $user->email = $request->email;
         $user->permissions = $request->permissions;
         if($request->password){
           $user->password = Hash::make($request->password);
         }
         $user->save();
         
         
        return response()->json([
            'status' => 'success',
            "message" => "Record Updated Successfull",
        ],200);
        
        
    }
    
    
    /*
     * Get Auth Details
     */
    public function del($id)
    {
        
    
          $user = User::find($id);
          
          if(!$user){
                 return response()->json([
                    'status' => 'error',
                    "message" => 'User Not Found',
                ],403);
          }
        
          
        $user->delete();
          
        return response()->json([
            'status' => 'success',
            "message" => "Record Deleted Successfull",
        ],200);
        
        
    }
    
    
    

     /*
     * Get Auth Details
     */
    public function auth(Request $request)
    {
        try {

            $data = $request->user();
            return response()->json([
                "message"=> "Get Profile Successfully",
                "user"=> $data,
                ],200);  
            
        } catch (\Exception $e) {
            return response()->json(["message" => "Error Found"],500);
        }
    }
    
   
    
    public function auth_update(Request $request){
        
         $user = $request->user();
        
         //  $validator = Validator::make($request->all(),[
        //     'name' =>'required|min:6|max:255|unique:users,name,'.$user->id, 
        //     'email' =>'required|email|unique:users,email,'.$user->id, 
        //  ]);
        
        //  if($validator->fails()){
        //     return response()->json(["validations"=>$validator->messages()],403);
        //  }
         
        //  $user->name = $request->name;
        //  $user->email = $request->email;
        //  $user->save();
         
        //  if($request->has('password')){
             
        //     $validator = Validator::make($request->all(),[
        //       'password' =>'required|min:6|max:255', 
        //     ]);
             
        //     if($validator->fails()){
        //        return response()->json($validator->messages(),403);
        //     }
             
        //     $user->password = Hash::make($request->password);
        //     $user->save();
        //  }
         
        //   if($request->has('thumbnail')){
        //         $path = '/images/profiles';
        //         $fileName = $user->id.'.'.$request->file('thumbnail')->extension(); 
        //         $request->file('thumbnail')->move(public_path($path), $fileName);
        //         $user->thumbnail = url('/public/images/profiles/'.$fileName);
        //         $user->save();
        //  }
         
        return response()->json([
            "message"=> "Update User Successfully",
            "user"=> $user,
        ],200);
        
    }
    



     /*
     * Get Auth Details
     */
    public function logout(Request $request)
    {
        try {

            $request->user()->currentAccessToken()->delete();
            return response()->json([
                "message"=> "Logout User Successfully",
                ],200);

        } catch (\Exception $e) {
            return response()->json(["message" => "Error Found"],500);
        }
        
    }
   

}